@@ -8,16 +8,6 @@ from adam.core.urdf_tree import URDFTree
 import dataclasses
 
 
-@dataclasses.dataclass
-class CustomInertia:
-    ixx: float
-    ixy: float
-    ixz: float
-    iyy: float
-    iyz: float
-    izz: float
-
-
 class RBDAlgorithms(SpatialMath):
     """This is a small abstract class that implements Rigid body algorithms retrieving robot quantities represented
     in mixed representation, for Floating Base systems - as humanoid robots.
@@ -101,6 +91,23 @@ class RBDAlgorithms(SpatialMath):
                     joint_i.axis[1],
                     joint_i.axis[2],
                 )
+            elif joint_i.type in ["prismatic"]:
+                q = joint_positions[joint_i.idx] if joint_i.idx is not None else 0.0
+                X_J = self.X_prismatic_joint(
+                    joint_i.origin.xyz,
+                    joint_i.origin.rpy,
+                    joint_i.axis,
+                    q,
+                )
+                X_p[i] = X_J
+                Phi[i] = self.vertcat(
+                    joint_i.axis[0],
+                    joint_i.axis[1],
+                    joint_i.axis[2],
+                    0,
+                    0,
+                    0,
+                )
 
         for i in range(self.tree.N - 1, -1, -1):
             link_i = self.tree.links[i]
@@ -165,16 +172,10 @@ class RBDAlgorithms(SpatialMath):
         return M, Jcm
 
     def extract_link_properties(self, link_i):
-        if link_i.inertial is not None:
-            I = link_i.inertial.inertia
-            mass = link_i.inertial.mass
-            o = link_i.inertial.origin.xyz
-            rpy = link_i.inertial.origin.rpy
-        else:
-            I = CustomInertia(0.0, 0.0, 0.0, 0.0, 0.0, 0.0)
-            mass = 0.0
-            o = [0.0, 0.0, 0.0]
-            rpy = [0.0, 0.0, 0.0]
+        I = link_i.inertial.inertia
+        mass = link_i.inertial.mass
+        o = link_i.inertial.origin.xyz
+        rpy = link_i.inertial.origin.rpy
         return I, mass, o, rpy
 
     def forward_kinematics(
@@ -189,30 +190,39 @@ class RBDAlgorithms(SpatialMath):
         Returns:
             T_fk (npt.ArrayLike): The fk represented as Homogenous transformation matrix
         """
-        chain = self.robot_desc.get_chain(self.root_link, frame)
+        chain = self.robot_desc.get_chain(self.root_link, frame, links=False)
         T_fk = self.eye(4)
         T_fk = T_fk @ base_transform
         for item in chain:
-            if item in self.robot_desc.joint_map:
-                joint = self.robot_desc.joint_map[item]
-                if joint.type == "fixed":
-                    xyz = joint.origin.xyz
-                    rpy = joint.origin.rpy
-                    joint_frame = self.H_from_Pos_RPY(xyz, rpy)
-                    T_fk = T_fk @ joint_frame
-                if joint.type in ["revolute", "continuous"]:
-                    # if the joint is actuated set the value
-                    q_ = joint_positions[joint.idx] if joint.idx is not None else 0.0
-                    T_joint = self.H_revolute_joint(
-                        joint.origin.xyz,
-                        joint.origin.rpy,
-                        joint.axis,
-                        q_,
-                    )
-                    T_fk = T_fk @ T_joint
+            joint = self.robot_desc.joint_map[item]
+            if joint.type == "fixed":
+                xyz = joint.origin.xyz
+                rpy = joint.origin.rpy
+                joint_frame = self.H_from_Pos_RPY(xyz, rpy)
+                T_fk = T_fk @ joint_frame
+            if joint.type in ["revolute", "continuous"]:
+                # if the joint is actuated set the value
+                q_ = joint_positions[joint.idx] if joint.idx is not None else 0.0
+                T_joint = self.H_revolute_joint(
+                    joint.origin.xyz,
+                    joint.origin.rpy,
+                    joint.axis,
+                    q_,
+                )
+                T_fk = T_fk @ T_joint
+            elif joint.type in ["prismatic"]:
+                # if the joint is actuated set the value
+                q_ = joint_positions[joint.idx] if joint.idx is not None else 0.0
+                T_joint = self.H_prismatic_joint(
+                    joint.origin.xyz,
+                    joint.origin.rpy,
+                    joint.axis,
+                    q_,
+                )
+                T_fk = T_fk @ T_joint
         return T_fk
 
-    def jacobian(
+    def joints_jacobian(
         self, frame: str, base_transform: npt.ArrayLike, joint_positions: npt.ArrayLike
     ) -> npt.ArrayLike:
         """Returns the Jacobian relative to the specified frame
@@ -223,44 +233,62 @@ class RBDAlgorithms(SpatialMath):
             joint_positions (npt.ArrayLike): The joints position
 
         Returns:
-            J_tot (npt.ArrayLike): The Jacobian relative to the frame
+            J (npt.ArrayLike): The Joints Jacobian relative to the frame
         """
-        chain = self.robot_desc.get_chain(self.root_link, frame)
-        T_fk = self.eye(4)
-        T_fk = T_fk @ base_transform
+        chain = self.robot_desc.get_chain(self.root_link, frame, links=False)
+        T_fk = self.eye(4) @ base_transform
         J = self.zeros(6, self.NDoF)
         T_ee = self.forward_kinematics(frame, base_transform, joint_positions)
         P_ee = T_ee[:3, 3]
         for item in chain:
-            if item in self.robot_desc.joint_map:
-                joint = self.robot_desc.joint_map[item]
-                if joint.type == "fixed":
-                    xyz = joint.origin.xyz
-                    rpy = joint.origin.rpy
-                    joint_frame = self.H_from_Pos_RPY(xyz, rpy)
-                    T_fk = T_fk @ joint_frame
-                if joint.type in ["revolute", "continuous"]:
-                    q_ = joint_positions[joint.idx] if joint.idx is not None else 0.0
-                    T_joint = self.H_revolute_joint(
-                        joint.origin.xyz,
-                        joint.origin.rpy,
-                        joint.axis,
-                        q_,
-                    )
-                    T_fk = T_fk @ T_joint
-                    p_prev = P_ee - T_fk[:3, 3].array
-                    z_prev = T_fk[:3, :3] @ joint.axis
-                    # J[:, joint.idx] = self.vertcat(
-                    #     cs.jacobian(P_ee, joint_positions[joint.idx]), z_prev) # using casadi jacobian
-                    if joint.idx is not None:
-                        J[:, joint.idx] = self.vertcat(
-                            self.skew(z_prev) @ p_prev, z_prev
-                        )
+            joint = self.robot_desc.joint_map[item]
+            if joint.type == "fixed":
+                xyz = joint.origin.xyz
+                rpy = joint.origin.rpy
+                joint_frame = self.H_from_Pos_RPY(xyz, rpy)
+                T_fk = T_fk @ joint_frame
+            if joint.type in ["revolute", "continuous"]:
+                q_ = joint_positions[joint.idx] if joint.idx is not None else 0.0
+                T_joint = self.H_revolute_joint(
+                    joint.origin.xyz,
+                    joint.origin.rpy,
+                    joint.axis,
+                    q_,
+                )
+                T_fk = T_fk @ T_joint
+                p_prev = P_ee - T_fk[:3, 3].array
+                z_prev = T_fk[:3, :3] @ joint.axis
+                J_lin = self.skew(z_prev) @ p_prev
+                J_ang = z_prev
 
+            if joint.type in ["prismatic"]:
+                q_ = joint_positions[joint.idx] if joint.idx is not None else 0.0
+                T_joint = self.H_prismatic_joint(
+                    joint.origin.xyz,
+                    joint.origin.rpy,
+                    joint.axis,
+                    q_,
+                )
+                T_fk = T_fk @ T_joint
+                z_prev = T_fk[:3, :3] @ joint.axis
+                J_lin = z_prev
+                J_ang = self.zeros(3)
+
+            if joint.idx is not None:
+                J[:, joint.idx] = self.vertcat(J_lin, J_ang)
+
+        return J
+
+    def jacobian(
+        self, frame: str, base_transform: npt.ArrayLike, joint_positions: npt.ArrayLike
+    ) -> npt.ArrayLike:
+
+        J = self.joints_jacobian(frame, base_transform, joint_positions)
+        T_ee = self.forward_kinematics(frame, base_transform, joint_positions)
         # Adding the floating base part of the Jacobian, in Mixed representation
         J_tot = self.zeros(6, self.NDoF + 6)
         J_tot[:3, :3] = self.eye(3)
-        J_tot[:3, 3:6] = -self.skew((P_ee - base_transform[:3, 3]))
+        J_tot[:3, 3:6] = -self.skew((T_ee[:3, 3] - base_transform[:3, 3]))
         J_tot[:3, 6:] = J[:3, :]
         J_tot[3:, 3:6] = self.eye(3)
         J_tot[3:, 6:] = J[3:, :]
@@ -278,39 +306,8 @@ class RBDAlgorithms(SpatialMath):
         Returns:
             J (npt.ArrayLike): The Jacobian between the root and the frame
         """
-        chain = self.robot_desc.get_chain(self.root_link, frame)
         base_transform = self.eye(4).array
-        T_fk = self.eye(4)
-        T_fk = T_fk @ base_transform
-        J = self.zeros(6, self.NDoF)
-        T_ee = self.forward_kinematics(frame, base_transform, joint_positions)
-        P_ee = T_ee[:3, 3]
-        for item in chain:
-            if item in self.robot_desc.joint_map:
-                joint = self.robot_desc.joint_map[item]
-                if joint.type == "fixed":
-                    xyz = joint.origin.xyz
-                    rpy = joint.origin.rpy
-                    joint_frame = self.H_from_Pos_RPY(xyz, rpy)
-                    T_fk = T_fk @ joint_frame
-                if joint.type in ["revolute", "continuous"]:
-                    q = joint_positions[joint.idx] if joint.idx is not None else 0.0
-                    T_joint = self.H_revolute_joint(
-                        joint.origin.xyz,
-                        joint.origin.rpy,
-                        joint.axis,
-                        q,
-                    )
-                    T_fk = T_fk @ T_joint
-                    p_prev = P_ee - T_fk[:3, 3]
-                    z_prev = T_fk[:3, :3] @ joint.axis
-                    # J[:, joint.idx] = self.vertcat(
-                    #     cs.jacobian(P_ee, joint_positions[joint.idx]), z_prev) # using casadi jacobian
-                    if joint.idx is not None:
-                        J[:, joint.idx] = self.vertcat(
-                            self.skew(z_prev) @ p_prev, z_prev
-                        )
-        return J
+        return self.joints_jacobian(frame, base_transform, joint_positions)
 
     def CoM_position(
         self, base_transform: npt.ArrayLike, joint_positions: npt.ArrayLike
@@ -396,7 +393,7 @@ class RBDAlgorithms(SpatialMath):
         )
         # set initial acceleration (rotated gravity + apparent acceleration)
         # reshape g as a vertical vector
-        a[0] = -X_to_mixed @ g.reshape(6, 1) + acc_to_mixed
+        a[0] = -X_to_mixed @ g.reshape(-1, 1) + acc_to_mixed
 
         for i in range(self.tree.N):
             link_i = self.tree.links[i]
@@ -431,6 +428,22 @@ class RBDAlgorithms(SpatialMath):
                     0, 0, 0, joint_i.axis[0], joint_i.axis[1], joint_i.axis[2]
                 )
                 v_J = Phi[i] * q_dot
+            elif joint_i.type in ["prismatic"]:
+                q = joint_positions[joint_i.idx] if joint_i.idx is not None else 0.0
+                q_dot = (
+                    joint_velocities[joint_i.idx] if joint_i.idx is not None else 0.0
+                )
+                X_J = self.X_prismatic_joint(
+                    joint_i.origin.xyz,
+                    joint_i.origin.rpy,
+                    joint_i.axis,
+                    q,
+                )
+                X_p[i] = X_J
+                Phi[i] = self.vertcat(
+                    joint_i.axis[0], joint_i.axis[1], joint_i.axis[2], 0, 0, 0
+                )
+                v_J = Phi[i] * q_dot
 
             if link_i.name == self.root_link:
                 v[i] = v_J
